@@ -15,13 +15,16 @@
  *
  */
 
+#define _GNU_SOURCE
 #include <stdbool.h>
 
 #include "emulator/bthost.h"
 #include "src/shared/util.h"
-
-#include "tester-main.h"
+#include "src/shared/tester.h"
+#include "src/shared/queue.h"
+#include "lib/bluetooth.h"
 #include "android/utils.h"
+#include "tester-main.h"
 
 static struct queue *list;
 
@@ -584,8 +587,7 @@ struct queue *get_avrcp_tests(void)
 	list = queue_new();
 
 	for (; i < sizeof(test_cases) / sizeof(test_cases[0]); ++i)
-		if (!queue_push_tail(list, &test_cases[i]))
-			return NULL;
+		queue_push_tail(list, &test_cases[i]);
 
 	return list;
 }
