@@ -10,9 +10,19 @@
 #include <ell/io.h>
 #include <ell/idle.h>
 #include <ell/main.h>
+#include <ell/settings.h>
 #include <ell/strv.h>
 #include <ell/string.h>
 #include <ell/utf8.h>
 #include <ell/dbus.h>
 #include <ell/dbus-service.h>
 #include <ell/dbus-client.h>
+#include <ell/key.h>
+#include <ell/cert.h>
+#include <ell/pem.h>
+#include <ell/base64.h>
+#include <ell/pkcs5.h>
+#include <ell/asn1-private.h>
+#include <ell/pkcs5-private.h>
+#include <ell/cert-private.h>
+#include <ell/pem-private.h>
