@@ -26,6 +26,7 @@
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -33,13 +34,13 @@
 #include <inttypes.h>
 
 #include "lib/bluetooth.h"
+#include "lib/uuid.h"
 
 #include "src/shared/util.h"
 #include "bt.h"
 #include "packet.h"
 #include "display.h"
 #include "l2cap.h"
-#include "uuid.h"
 #include "keys.h"
 #include "sdp.h"
 #include "avctp.h"
@@ -1965,7 +1966,7 @@ static bool avrcp_folder_item(struct avctp_frame *avctp_frame,
 	print_field("%*cNameLength: 0x%04x (%u)", indent, ' ',
 					namelen, namelen);
 
-	print_field("%*cName: ", indent, ' ');
+	printf("%*cName: ", indent+8, ' ');
 	for (; namelen > 0; namelen--) {
 		uint8_t c;
 		if (!l2cap_frame_get_u8(frame, &c))
@@ -1973,6 +1974,7 @@ static bool avrcp_folder_item(struct avctp_frame *avctp_frame,
 
 		printf("%1c", isprint(c) ? c : '.');
 	}
+	printf("\n");
 
 	return true;
 }
@@ -1984,8 +1986,7 @@ static bool avrcp_attribute_entry_list(struct avctp_frame *avctp_frame,
 
 	for (; count > 0; count--) {
 		uint32_t attr;
-		uint16_t charset;
-		uint8_t len;
+		uint16_t charset, len;
 
 		if (!l2cap_frame_get_be32(frame, &attr))
 			return false;
@@ -1999,13 +2000,13 @@ static bool avrcp_attribute_entry_list(struct avctp_frame *avctp_frame,
 		print_field("%*cCharsetID: 0x%04x (%s)", indent, ' ',
 						charset, charset2str(charset));
 
-		if (!l2cap_frame_get_u8(frame, &len))
+		if (!l2cap_frame_get_be16(frame, &len))
 			return false;
 
-		print_field("%*cAttributeLength: 0x%02x (%u)", indent, ' ',
+		print_field("%*cAttributeLength: 0x%04x (%u)", indent, ' ',
 						len, len);
 
-		print_field("%*cAttributeValue: ", indent, ' ');
+		printf("%*cAttributeValue: ", indent+8, ' ');
 		for (; len > 0; len--) {
 			uint8_t c;
 
@@ -2014,6 +2015,7 @@ static bool avrcp_attribute_entry_list(struct avctp_frame *avctp_frame,
 
 			printf("%1c", isprint(c) ? c : '.');
 		}
+		printf("\n");
 	}
 
 	return true;
@@ -2051,7 +2053,7 @@ static bool avrcp_media_element_item(struct avctp_frame *avctp_frame,
 	print_field("%*cNameLength: 0x%04x (%u)", indent, ' ',
 					namelen, namelen);
 
-	print_field("%*cName: ", indent, ' ');
+	printf("%*cName: ", indent+8, ' ');
 	for (; namelen > 0; namelen--) {
 		uint8_t c;
 		if (!l2cap_frame_get_u8(frame, &c))
@@ -2059,6 +2061,7 @@ static bool avrcp_media_element_item(struct avctp_frame *avctp_frame,
 
 		printf("%1c", isprint(c) ? c : '.');
 	}
+	printf("\n");
 
 	if (!l2cap_frame_get_u8(frame, &count))
 		return false;
@@ -2317,9 +2320,9 @@ static bool avrcp_get_folder_items(struct avctp_frame *avctp_frame)
 						count, count);
 
 	for (; count > 0; count--) {
-		uint16_t attr;
+		uint32_t attr;
 
-		if (!l2cap_frame_get_be16(frame, &attr))
+		if (!l2cap_frame_get_be32(frame, &attr))
 			return false;
 
 		print_field("%*cAttributeID: 0x%08x (%s)", indent, ' ',
