@@ -24,10 +24,13 @@
 
 #define AUDIO_CONTROL_INTERFACE "org.bluez.MediaControl1"
 
-struct control *control_init(struct audio_device *dev, GSList *uuids);
-void control_update(struct control *control, GSList *uuids);
-void control_unregister(struct audio_device *dev);
-gboolean control_is_active(struct audio_device *dev);
+struct btd_service;
 
-int control_connect(struct audio_device *dev);
-int control_disconnect(struct audio_device *dev);
+int control_init_target(struct btd_service *service);
+int control_init_remote(struct btd_service *service);
+void control_unregister(struct btd_service *service);
+
+int control_connect(struct btd_service *service);
+int control_disconnect(struct btd_service *service);
+
+int control_set_player(struct btd_service *service, const char *path);
