@@ -58,9 +58,6 @@ struct queue *queue_new(void)
 	struct queue *queue;
 
 	queue = new0(struct queue, 1);
-	if (!queue)
-		return NULL;
-
 	queue->head = NULL;
 	queue->tail = NULL;
 	queue->entries = 0;
@@ -78,35 +75,14 @@ void queue_destroy(struct queue *queue, queue_destroy_func_t destroy)
 	queue_unref(queue);
 }
 
-static struct queue_entry *queue_entry_ref(struct queue_entry *entry)
-{
-	if (!entry)
-		return NULL;
-
-	__sync_fetch_and_add(&entry->ref_count, 1);
-
-	return entry;
-}
-
-static void queue_entry_unref(struct queue_entry *entry)
-{
-	if (__sync_sub_and_fetch(&entry->ref_count, 1))
-		return;
-
-	free(entry);
-}
-
 static struct queue_entry *queue_entry_new(void *data)
 {
 	struct queue_entry *entry;
 
 	entry = new0(struct queue_entry, 1);
-	if (!entry)
-		return NULL;
-
 	entry->data = data;
 
-	return queue_entry_ref(entry);
+	return entry;
 }
 
 bool queue_push_tail(struct queue *queue, void *data)
@@ -117,8 +93,6 @@ bool queue_push_tail(struct queue *queue, void *data)
 		return false;
 
 	entry = queue_entry_new(data);
-	if (!entry)
-		return false;
 
 	if (queue->tail)
 		queue->tail->next = entry;
@@ -141,8 +115,6 @@ bool queue_push_head(struct queue *queue, void *data)
 		return false;
 
 	entry = queue_entry_new(data);
-	if (!entry)
-		return false;
 
 	entry->next = queue->head;
 
@@ -176,8 +148,6 @@ bool queue_push_after(struct queue *queue, void *entry, void *data)
 		return false;
 
 	new_entry = queue_entry_new(data);
-	if (!new_entry)
-		return false;
 
 	new_entry->next = qentry->next;
 
@@ -208,7 +178,7 @@ void *queue_pop_head(struct queue *queue)
 
 	data = entry->data;
 
-	queue_entry_unref(entry);
+	free(entry);
 	queue->entries--;
 
 	return data;
@@ -246,14 +216,8 @@ void queue_foreach(struct queue *queue, queue_foreach_func_t function,
 	while (entry && queue->head && queue->ref_count > 1) {
 		struct queue_entry *next;
 
-		queue_entry_ref(entry);
-
-		function(entry->data, user_data);
-
 		next = entry->next;
-
-		queue_entry_unref(entry);
-
+		function(entry->data, user_data);
 		entry = next;
 	}
 	queue_unref(queue);
@@ -302,7 +266,7 @@ bool queue_remove(struct queue *queue, void *data)
 		if (!entry->next)
 			queue->tail = prev;
 
-		queue_entry_unref(entry);
+		free(entry);
 		queue->entries--;
 
 		return true;
@@ -316,9 +280,12 @@ void *queue_remove_if(struct queue *queue, queue_match_func_t function,
 {
 	struct queue_entry *entry, *prev = NULL;
 
-	if (!queue || !function)
+	if (!queue)
 		return NULL;
 
+	if (!function)
+		function = direct_match;
+
 	entry = queue->head;
 
 	while (entry) {
@@ -335,7 +302,7 @@ void *queue_remove_if(struct queue *queue, queue_match_func_t function,
 
 			data = entry->data;
 
-			queue_entry_unref(entry);
+			free(entry);
 			queue->entries--;
 
 			return data;
@@ -386,7 +353,7 @@ unsigned int queue_remove_all(struct queue *queue, queue_match_func_t function,
 			if (destroy)
 				destroy(tmp->data);
 
-			queue_entry_unref(tmp);
+			free(tmp);
 			count++;
 		}
 	}
