@@ -29,11 +29,11 @@
 #include <stdio.h>
 #include <sys/socket.h>
 
-#include <bluetooth/bluetooth.h>
-#include <bluetooth/hci.h>
-#include <bluetooth/hci_lib.h>
+#include "lib/bluetooth.h"
+#include "lib/hci.h"
+#include "lib/hci_lib.h"
 
-int main(void)
+int main(int argc, char *argv[])
 {
 	uint32_t type_mask;
 	uint32_t event_mask[2];
